# Code written by: 
# Juho Kopra / juho.j.kopra@jyu.fi / University of Jyväskylä, Finland / 20th April 2017

# parameter values for simulation are based on one earlier run with true data
load(file="parameters_for_simulation_29032017.Rdata",verbose = T) # original run: v8run2siml

# then load the actual data
load(file=paste(getwd(),"/runs/prerun.Rdata",sep="",collapse=""), verbose = TRUE)

#id_rank
store<-setdiff(ls(),"id_rank")
# expit function is needed later
expit <- function(x) { exp(x)/(1+exp(x)); }

Ntotal<-dataList$Ntotal
pick2<-dataList$pick2 # group by area, sex, study year, indexed from 1 to 32
age0<-dataList$age0
age1<-dataList$age1
sex<-dataList$sex
year<-dataList$year
area<-dataList$area

# 1) simulate smoking
ii<-1:Ntotal
psmoke<-expit(parametres[paste("beta_cohort_birth[",pick2[ii],"]",sep="")] + parametres[paste("beta_cohort_smoke[",pick2[ii],"]",sep="")]*age0[ii])
SMOKE<-rbinom(length(psmoke),1,psmoke)

# 2) simulate participation
ppart<-expit(parametres[paste("beta0[",sex,",",SMOKE[ii]+1,",",year[ii],"]",sep="")] + parametres[paste("betaAge[",sex[ii],",",SMOKE[ii]+1,"]",sep="")]*age0[ii])
part<-rbinom(length(ppart),1,ppart)

library(plyr)

# 3) simulate survival model
betaSMOKE<-parametres[c("betaSMOKE[1]","betaSMOKE[2]")]

yearfull<-1972+(dataList$year-1)*5
followuplength<-floor(pmin(2012-yearfull,age1)) # age1 koettaa sanoa, että jos kuoli niin sitten on vähän lyhyempi!
age0<-floor(age0+45)
censoring_age<-age0+followuplength-1 -2# viimeinen ikäyearfull, jonka henkilö on riskin alla!

# load pp, which is a baseline hasard for non-smokers:
pp<-array(NA,dim=c(2,74))
pp[1,]<-parametres[paste("pp[",1,",",1:74,"]",sep="")]
pp[2,]<-parametres[paste("pp[",2,",",1:74,"]",sep="")]
betaSMOKE<-parametres[c("betaSMOKE[1]","betaSMOKE[2]")]

nareas<-length(unique(dataList$area))

sim.age<-function(alive0,prob) {
    res<-NULL;
    if (length(alive0)==length(prob)) {
        sim<-rbinom(length(prob),1,alive0*prob)
        res<-ifelse(sum(sim)>0,min(which(sim==1)),NA)
    } else {
        res <- "err"
    }
    res
}

# dNalt_sim
alive0alt_sim<-dNalt_sim<-array(0,dim=c(2,nareas,2,74))
# for persons who have sex==1 & SMOKE==0 (non-smoking men) and who participated
indexo<-which(part==1) #for participants

for (setSMOKE in 0:1) {
    for (setSEX in 1:2) {
        for (setAREA in 1:nareas) {
            sel<-sex==setSEX & SMOKE==setSMOKE & area==setAREA & part==1
            ageind<-1:74 # add 24 so you get the actual age
            prob<-pmin(exp(betaSMOKE[setSEX]*setSMOKE)*pp[setSEX,ageind],1)

            sel.i<-which(sel)
            k<-sel.i[1]
            event_age<-rep(NA,sum(sel))
            alive0k<-c(rep(0,age0[k]-24-1),rep(1,censoring_age[k]-age0[k]+1),rep(0,98-censoring_age[k]))
            event_age[1]<-sim.age(alive0=alive0k,prob=prob)
            ki<-2
            for(k in sel.i[-1]) {
                #alive0i: tells when individual "sel.i" is in risk group!
                alive0i <- c(rep(0,age0[k]-24-1),rep(1,censoring_age[k]-age0[k]+1),rep(0,98-censoring_age[k]))
                alive0k <- alive0k + alive0i
                alive0i*prob
                event_age[ki]<-sim.age(alive0=alive0i,prob=prob)
                ki=ki+1
            }
#            alive0k*prob
            sens<-is.na(event_age)
#            table(event_age)
            tab<-table(event_age[!sens])
            # We have simulated alive0k and survival times. Next we need to build dNalt_sim.
            alive0alt_sim[setSEX,setAREA,setSMOKE+1,ageind] <- alive0k
            dNalt_sim[setSEX,setAREA,setSMOKE+1,as.integer(names(tab))] <- tab
            dNalt_sim[setSEX,setAREA,setSMOKE+1,-as.integer(names(tab))]<-0
        }
    }
}

# Now repeat the same for non-participants!

# dNnp_sim
alive0np_sim<-dNnp_sim<-array(0,dim=c(sum(part==0),74))
indexk <- which(part==0)
alive0np_sim[1,] <- c(rep(1,censoring_age[indexk[1]]-24+1),rep(0,98-censoring_age[indexk[1]]-1))
for(k in 2:length(indexk)) {
    alive0np_sim[k,] <- c(rep(0,age0[indexk[k]]-24-1),rep(1,censoring_age[indexk[k]]-age0[indexk[k]]+1),rep(0,98-censoring_age[indexk[k]]))
}
colnames(alive0np_sim)<-25:98
sel<- part==0
event_age<-vector("double",sum(sel))

# for persons who have sex==1 & SMOKE==0 (non-smoking men) and who participated
# "i_nonpart" loops through the selected persons 1..83132
# "i" loops through all persons 1..19343
i <- 1
for (i_nonpart in which(part==0)) {
    ageind<-age0[i_nonpart]:98
    prob<-pmin(exp(betaSMOKE[sex[i_nonpart]]*SMOKE[i_nonpart])*pp[sex[i_nonpart],],1)
    names(prob)<-25:98
    
    ifelse(any(abs(prob-0.5)>0.5),cat("!"),"")
    sim<-rbinom(length(alive0np_sim[i,]),1,prob*alive0np_sim[i,])
    simulaatio<-which(1==sim)
    
    event_age[i]<-ifelse(any(simulaatio>0),24+min(simulaatio),NA) # if NA is observed: censoring event = no disease event
    if(event_age[i]>censoring_age[i_nonpart] & !is.na(event_age[i])) {
        print(paste("There was something wrong in simulation. Non-participant index: i=",i,sep=""))
    }

    # Now we need to take into account what age person was when leaving the follow-up!
    
    # We have simulated alive0k and survival times. Next we need to build dNnp_sim.
    if(!is.na(event_age[i]) & event_age[i]<=censoring_age[i_nonpart]) {
        dNnp_sim[i,event_age[i]-24]<-1
    }
    i<-i+1
}
table(event_age,useNA="always")
# check if there was problems with simulation
table(event_age>censoring_age[which(part==0)] & !is.na(event_age)) # 0 persons with problems with simulation!
table(event_age==censoring_age[which(part==0)] & !is.na(event_age)) # couple of events take place at the moment of censoring, which is fine.

# non-smoking men
check<-rbind(colSums(dNnp_sim) + colSums(dNalt_sim[1,,1,]),
             colSums(alive0np_sim) + colSums(alive0alt_sim[1,,1,])
       )


#men
check_smoke_m<-rbind(colSums(dNalt_sim[1,,2,]),
    colSums(alive0alt_sim[1,,2,])
)
check_nosmoke_m<-rbind(
    colSums(dNalt_sim[1,,1,]),
    colSums(alive0alt_sim[1,,1,])
)
plot(25:98,check_smoke_m[1,]/check_smoke_m[2,],col="red",main="Men",xlab="age",ylab="proportion")
points(25:98,check_nosmoke_m[1,]/check_nosmoke_m[2,],col="blue")

check_np_m<-rbind(colSums(dNnp_sim[sex[which(part==0)]==1,]),
                  colSums(alive0np_sim[sex[which(part==0)]==1,]))
points(25:98,check_np_m[1,]/check_np_m[2,],col="magenta")

#women
check_smoke_w<-rbind(
    colSums(dNalt_sim[2,,2,]),
    colSums(alive0alt_sim[2,,2,])
)
check_nosmoke_w<-rbind(
    colSums(dNalt_sim[2,,1,]),
    colSums(alive0alt_sim[2,,1,])
)
plot(25:98,check_smoke_w[1,]/check_smoke_w[2,],col="red",ylim=c(0,0.1),main="Women",xlab="age",ylab="proportion")
points(25:98,check_nosmoke_w[1,]/check_nosmoke_w[2,],col="blue")

check_np_w<-rbind(colSums(dNnp_sim[sex[which(part==0)]==2,]),
                  colSums(alive0np_sim[sex[which(part==0)]==2,]))
points(25:98,check_np_w[1,]/check_np_w[2,],col="magenta")

table(SMOKE[part==0],sex[part==0])/sum(table(SMOKE[part==0],sex[part==0]))
table(SMOKE[part==1],sex[part==1])/sum(table(SMOKE[part==1],sex[part==1]))

1-table(SMOKE[part==0],sex[part==0])/(table(SMOKE[part==0],sex[part==0])+table(SMOKE[part==1],sex[part==1]))
## Alltogether we simulated:
#alive0alt_sim # participants
#dNalt_sim     # participants
#alive0np_sim  # non-participants
#dNnp_sim      # non-participants
SMOKEtrue<-SMOKE # 
SMOKE[part==0]<-NA

# and then change following data:
# i.sex_np
dataList$i.sex_np<-dataList$i.sex[is.na(SMOKE)] # i.sex -nimistä ei ole => siis käyttää i.sex_np-muuttujaa!!!
length(dataList$i.sex_np)
table(dataList$i.sex_np)
# i.area_np
dataList$i.area_np<-dataList$i.area[is.na(SMOKE)]
length(dataList$i.area_np)
# Ntotal_np
dataList$Ntotal_np <- sum(is.na(SMOKE))
# inp
dataList$inp<-c(1:Ntotal)[is.na(SMOKE)]
length(dataList$inp)

# check that simulated data have correct dimensions
dim(dataList$alive0alt);dim(alive0alt_sim)
dataList$alive0alt<-alive0alt_sim
dim(dataList$dNalt);dim(dNalt_sim)
dataList$dNalt<-dNalt_sim
dim(dataList$alive0_np);dim(alive0np_sim)
dataList$alive0_np<-alive0np_sim
dim(dataList$dN_np);dim(dNnp_sim) # dimensions of these can be different because as we simulate data, the number of censored event is likely to change
dataList$dN_np<-dNnp_sim
dataList$Ntotal_np<-dim(dNnp_sim)[1]
dataList$SMOKE<-SMOKE
dataList$SMOKEtrue<-SMOKEtrue
dataList$part<-part

r<-grep("pp",names(parametres))

par(mfrow=c(1,2))
plot(parametres[r[seq(1,length(r),2)]],col="red",lwd=2,main="Men",ylab="proportion",xlab="age-25")
cat(paste(colSums(dNalt_sim[1,,1,]) / colSums(alive0alt_sim[1,,1,]),collapse=", "),file="testmen.txt",sep="\n",append=TRUE)
lines(colSums(dNalt_sim[1,,1,]) / colSums(alive0alt_sim[1,,1,]),col="green")

plot(parametres[r[seq(2,length(r),2)]],col="red",lwd=2,main="Women",ylab="proportion",xlab="age-25")
cat(paste(colSums(dNalt_sim[2,,1,]) / colSums(alive0alt_sim[2,,1,]),collapse=", "),file="testwomen.txt",sep="\n",append=TRUE)
lines(colSums(dNalt_sim[2,,1,]) / colSums(alive0alt_sim[2,,1,]),col="green")

par(mfrow=c(1,2))
tm<-read.csv2(file="testmen.txt",sep=",",header=F)
tm<-as.matrix(tm)
tm<-apply(tm,2,as.numeric)
plot(parametres[r[seq(1,length(r),2)]],tm[1,],col="green",type="l",main="Miehet",ylim=c(0,0.5))
for(i in 2:dim(tm)[1]) {
    lines(parametres[r[seq(1,length(r),2)]],tm[i,],col="green")
}
lines(seq(0,1,0.01),seq(0,1,0.01))
tn<-read.csv2(file="testwomen.txt",sep=",",header=F)
tn<-as.matrix(tn)
tn<-apply(tn,2,as.numeric)
plot(parametres[r[seq(2,length(r),2)]],tn[1,],col="green",type="l",main="Naiset")
for(i in 2:dim(tn)[1]) {
    lines(parametres[r[seq(2,length(r),2)]],tn[i,],col="green")
}
lines(seq(0,1,0.01),seq(0,1,0.01))


save(list=store,file=paste("./runs/prerun_simul.Rdata",sep="",collapse=""))

