# Code written by: 
# Juho Kopra / juho.j.kopra@jyu.fi / University of Jyväskylä, Finland / 20th April 2017
rm(list=ls())
setwd("/set/your/own/path/kopra_etal_codes/") # Set a local folder

maxId<-7      # number of MCMC chains
maxYear<-2007 # the latest year to include in the trend plots

# LOAD RESULTS DATA (MCMC RUN) #
load(file="results_out.Rdata",verbose=T)

### COMPUTE THE BAYES TRENDS ###
id<-1
par.samples.final2<-Reduce(cbind,par.samples.final)
maxindex<-32
sumsmoke<-par.samples.final2[,paste("sumsmoke[",1:maxindex,"]",sep="")]
sumnosmoke<-par.samples.final2[,paste("sumnosmoke[",1:maxindex,"]",sep="")]

propsmoke<-sumsmoke/(sumsmoke+sumnosmoke)
quantiles<-as.data.frame(t(apply(propsmoke,2,FUN=function(x) {quantile(x,prob=c(0.025,0.975))})))

load(file=paste(c("./runs/prerun_simul.Rdata"),collapse=""),verbose = TRUE)
names1<-vector("character", maxindex)
for(i in 1:maxindex) {
    names1[i]<-names(dataList$pick2[which(dataList$pick2==i)[1]])
}
df_trends<-Reduce(rbind,strsplit(names1," & "))
df_trends<-apply(df_trends,2,"as.integer")
row.names(df_trends)<-names1
colnames(df_trends)<-c("sex","year","studyarea")
df_trends<-cbind(df_trends,quantiles)
df_trends$mean <- apply(propsmoke,2,FUN=mean)
df_trends$trendtype<-"bayestrend"
### THE BAYES TRENDS ARE NOW LOADED ###


years<-c(1972,2007)
studyareanames<-c(NA,"North Karelia","Northern Savonia")
legend.x<-c(1983,1972,1972,1972)
legend.y<-c(0.55,0.3,0.3,0.3)

xlab<-""

#-------------------------------------
# 19.4.2017
load(file="./datas/frdata_for_JAGS_20042017.Rdata",verbose=TRUE)
frdata<-subset(frdata,subset=c(year<=2007 & age1>age0 & age0 <=65 & (area<=3 | is.na(area))))
frdata$SMOKEtrue<-dataList$SMOKEtrue
require(plyr)
df_part<-ddply(frdata[!is.na(frdata$smoking),],c("area","sex","year"),function(x) {c("mean(true)"=mean(x$SMOKEtrue,na.rm=T),"mean"=mean(x$smoking==3,na.rm=T),"sumsmoke"=sum(x$smoking==3,na.rm=T),"sumnosmoke"=sum(x$smoking!=3,na.rm=T));})
df_part$trendtype<-"participants"
df_part$"2.5%"<-qbeta(c(0.025),df_part$sumsmoke,df_part$sumnosmoke)
df_part$"97.5%"<-qbeta(c(0.975),df_part$sumsmoke,df_part$sumnosmoke)
names(df_part)<-c("studyarea","sex","year","mean(true)","mean","sumsmoke","sumnosmoke","trendtype","2.5%","97.5%")
df_true<-df_part[,c("studyarea","sex","year","mean(true)")]
df_true$trendtype="true"
df_true$"2.5%"<-df_true$"97.5%"<-NA
names(df_true)[4]<-"mean"
df_trends<-rbind(rbind(df_trends,df_part[,c("studyarea","sex","year","mean","trendtype","2.5%","97.5%")]),df_true)
#-------------------------------------

pdf(file="plot_simulation.pdf",width=8,height=8)
par(mfrow=c(2,2),mai=c(0.7,0.9,0.5,0.3),mar=c(2,4,2,2))
pchs<-c(15,20,17) # black, blue, red
for(area in 2:3) {
    # men:
    sub_total<-subset(df_trends,subset=c(studyarea==area & sex==1 & year<=2007 & trendtype=="bayestrend"))
    plot(sub_total$year,sub_total$mean,type="b",pch=pchs[1],ylim=c(0.2,0.55),main=paste(studyareanames[area],"men"),xlim=years,ylab="proportion of smokers",xlab=xlab,axes=F)
    
    sub_part<-subset(df_trends,subset=c(studyarea==area & sex==1 & year<=2007 & trendtype=="participants"))
    lines(sub_part$year,sub_part$mean,type="b",col="blue",lwd=2,pch=pchs[2])
    lines(sub_total$year,sub_total$"2.5%",type="l",lty=2)
    lines(sub_total$year,sub_total$"97.5%",type="l",lty=2)
    
    sub_true<-subset(df_trends,subset=c(studyarea==area & sex==1 & year<=2007 & trendtype=="true"))
    lines(sub_true$year,sub_true$mean,type="b",col="red",lwd=2,pch=pchs[3])
    
    box()
    axis(side=1,tck=-0.03,at=seq(1972,2007,5)) # labels = NA prevents the creation of the numbers and tick marks, tck is how long the tick mark is.
    axis(side=2,tck=-0.03,at=seq(0.2,0.55,0.05))
    
    # women:
    sub_total<-subset(df_trends,subset=c(studyarea==area & sex==2 & year<=2007 & trendtype=="bayestrend"))
    plot(sub_total$year,sub_total$mean,type="b",pch=pchs[1],ylim=c(0.05,0.3),main=paste(studyareanames[area],"women"),xlim=years,ylab="proportion of smokers",xlab=xlab,axes=F)
    
    sub_part<-subset(df_trends,subset=c(studyarea==area & sex==2 & year<=2007 & trendtype=="participants"))
    lines(sub_part$year,sub_part$mean,type="b",col="blue",lwd=2,pch=pchs[2])
    lines(sub_total$year,sub_total$"2.5%",type="l",lty=2)
    lines(sub_total$year,sub_total$"97.5%",type="l",lty=2)
    
    sub_true<-subset(df_trends,subset=c(studyarea==area & sex==2 & year<=2007 & trendtype=="true"))
    lines(sub_true$year,sub_true$mean,type="b",col="red",lwd=2,pch=pchs[3])
    
    if(area==2) {
      legend(1972,0.3,pch=c(pchs[c(2,3,1)],NA),
             col=c("blue","red","black","black"),lty=c(1,1,1,2),lwd=c(2,2,1,1),
             legend=c("participants only","true trends","Bayesian modelling","95 % credible interval"))
    }
    box()
    axis(side=1,tck=-0.03,at=seq(1972,2007,5))
    axis(side=2,tck=-0.03,at=seq(0.05,0.3,0.05))
}
dev.off()
# PLEASE NOTE: THE TRENDS ARE DIFFERENT THAN IN THE ARTICLE, BECAUSE THE DATA IS SIMULATED, NOT THE ORIGINAL (DUE TO COPYRIGHT REASONS).
