# Code written by: 
# Juho Kopra / juho.j.kopra@jyu.fi / University of Jyväskylä, Finland / 20th April 2017
  rm(list=ls())

  setwd("/set/your/own/path/kopra_etal_codes/") # Set a local folder
  
  # ----------------- COMMENTS ------------------- #
  # This code is a master file for executing a data simulation and making data ready for analysis. Model fitting is executed using other .R-file. 
  # requirements: R2jags -package (which requires JAGS to be installed)
  # ---------------------------------------------- #
  
  model_specific_folder<-"chain"
  never_rm<-c("model_name","date","minimal","prerun","model_specific_folder","never_rm","minimal_code_name","prerun_code_name","trend_building_code_name","id_rank")
  
  # Next build model specific folders
  {
    id_rank <- 1
    proposed<-paste(getwd(),"/runs/",model_specific_folder,"-",id_rank,sep="",collapse="")
    if (!dir.exists(proposed)) {
        dir.create(proposed)
    }
    if (!dir.exists(paste(proposed,"/images",sep="",collapse=""))) {
        dir.create(paste(proposed,"/images",sep="",collapse=""))
    }    
    
    model_specific_folder<-paste(model_specific_folder,"-",id_rank,sep="",collapse="")
  }

  # lets run a prerun code to make data ready for simulation
  {
    # lets run a prerun
    source("./codes/preruns/prerun_smokingdata_jags_model5c.r")
    rm(list=setdiff(ls(),never_rm))
    print(paste(c("id:",id_rank," Prerun finished."),collapse=" "))
  }

  # lets run simulation / (DON'T RUN AT CSC SERVER UNDER RMPI!)
  {
      source("./codes/simulation_code_final.r")
      rm(list=setdiff(ls(),never_rm))
      print(paste(c("id:",id_rank," Testing data prepared."),collapse=" "))
  }
  id_rank
  
  for (id_r in 2:7) { # remember to change if number of chains is changed.
      newfolder<-paste("./runs/chain-",id_r,sep="")
      if(!dir.exists(newfolder)) {
          dir.create(newfolder)
          dir.create(paste(c(newfolder,"images"),collapse="/"))
      }
      file.copy(c("./runs/chain-1/jags_survival_model_smoking.txt"), newfolder)
  }

  
